/**
 * Copyright 2007 by jFramework.org.
 */
package org.jframework.core.util;

/**
 * Basic implementation of a proxy.
 *
 * @author mgridnev
 * @version 1.0, Jun 21, 2007
 */
public abstract class AbstractProxy<T> implements Proxy<T>
{
    /**
     * Returns hash code
     */
    public int hashCode()
    {
        final Object object = getProxiedObject();

        return object == null ? 0 : object.hashCode();
    }

    /**
     * Compares for equality.
     */
    public boolean equals(Object other)
    {
        if (this == other) {
            return true;
        }

        if (!(other instanceof Proxy)) {
            return false;
        }

        final Object proxied = getProxiedObject();
        final Object proxiedOther = ((Proxy) other).getProxiedObject();

        return proxied == null && proxiedOther == null || proxied != null && proxied.equals(proxiedOther);
    }

    /**
     * Returns short description
     */
    public String toString()
    {
        final Object object = getProxiedObject();

        return object == null ? "null" : object.toString();
    }

    /**
     * Compares proxied objects.
     */
    @SuppressWarnings("unchecked")
    public int compareTo(Proxy<T> other)
    {
        return ((Comparable) getProxiedObject()).compareTo(ProxyUtils.unproxy(other));
    }
}
