/**
 * Copyright 2006-2007 by jFramework.org.
 */
package org.jframework.core.util;

/**
 * A value with an optional desciption.  The descibed value a used primarily in the UI for rendering. If the description is provided, it is returned from {@link #toString()},
 * otherwise the value's {@link #toString()} is used.  The described value delegates {@link #hashCode()} and {@link #equals(Object)} to the underlying value.
 *
 * @author mgridnev
 * @version 2.0, Dec 13, 2006
 */
public class DescribedValue<T> extends AbstractProxy<T>
{
    /**
     * The value
     */
    private final T value;

    /**
     * Description
     */
    private final String description;

    /**
     * Creates a value.
     *
     * @param value       the required value
     * @param description an optional description, may be <code>null</code> or an empty string.  If the description is provided, it is returned from {@link #toString()}, otherwise
     *                    the value's {@link #toString()} is used
     */
    public DescribedValue(T value, String description)
    {
        this.value = value;
        this.description = description;
    }

    /**
     * Returns description.
     */
    public String getDescription()
    {
        return description;
    }

    /**
     * Returns the object being proxied.
     */
    public T getProxiedObject()
    {
        return value;
    }

    /**
     * Returns string representation of the version
     */
    public String toString()
    {
        if (description != null && description.length() > 0) {
            return description;
        }

        return super.toString();
    }
}
