/**
 * Copyright 2007 by jFramework.org.
 */
package org.jframework.core.util;

import java.util.Collection;
import java.util.ArrayList;

import org.apache.commons.collections.Transformer;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.PredicateUtils;

/**
 * Proxy utilities.
 *
 * @author mgridnev
 * @version 1.0, Jun 21, 2007
 */
public abstract class ProxyUtils
{
    /**
     * Unproxying transformer.
     */
    public static final Transformer unproxingTransformer = new Transformer()
    {
        public Object transform(Object input)
        {
            return ProxyUtils.unproxy(input);
        }
    };

    /**
     * Creates a simple proxy returning the given value
     */
    public static <T> Proxy<T> create(final T value)
    {
        return new AbstractProxy<T>()
        {
            /**
             * Returns the object being proxied
             */
            public T getProxiedObject()
            {
                return value;
            }
        };
    }


    /**
     * Conditionally unproxies the object walking a chain or proxies
     *
     * @return an instance of {@link org.jframework.core.util.Proxy} or just a POJO
     */
    @SuppressWarnings("unchecked")
    public static final Object unproxy(Object obj)
    {
        // A single proxy
        if (obj instanceof Proxy<?>) {
            return unproxy(((Proxy<?>) obj).getProxiedObject());
        }

        // If we have a collection containing one or more proxies, make a copy and unproxy the appropriate objects
        if (obj instanceof Collection &&
            CollectionUtils.find((Collection) obj, PredicateUtils.instanceofPredicate(Proxy.class)) != null) {

            final ArrayList listUnproxied = new ArrayList((Collection) obj);
            CollectionUtils.transform(listUnproxied, unproxingTransformer);

            return listUnproxied;
        }

        // Nothing to unproxy
        return obj;
    }

}
